@@ -2,7 +2,7 @@
 # :Project:   python-rapidjson -- Benchmarks specific pytest configuration
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   MIT License
-# :Copyright: © 2016, 2017, 2018 Lele Gaifax
+# :Copyright: © 2016, 2017, 2018, 2020 Lele Gaifax
 #
 
 from collections import namedtuple
@@ -132,6 +132,24 @@ else:
                   ujson.loads),
     ])
 
+try:
+    import hyperjson
+except ImportError:
+    pass
+else:
+    contenders.append(Contender('hyperjson',
+                                hyperjson.dumps,
+                                hyperjson.loads))
+
+try:
+    import orjson
+except ImportError:
+    pass
+else:
+    contenders.append(Contender('orjson',
+                                orjson.dumps,
+                                orjson.loads))
+
 
 def pytest_generate_tests(metafunc):
     if 'contender' in metafunc.fixturenames:
