@@ -3,11 +3,11 @@
 # :Author:    Ken Robbins <ken@kenrobbins.com>
 # :License:   MIT License
 # :Copyright: © 2015 Ken Robbins
-# :Copyright: © 2015, 2016, 2017, 2018 Lele Gaifax
+# :Copyright: © 2015, 2016, 2017, 2018, 2019, 2020 Lele Gaifax
 #
 
 from calendar import timegm
-from datetime import date, datetime, time
+from datetime import date, datetime, time, timezone, timedelta
 import io
 import math
 import uuid
@@ -17,7 +17,6 @@ import pytest
 import rapidjson as rj
 
 
-@pytest.mark.unit
 def test_skipkeys():
     o = {True: False, -1: 1, 1.1: 1.1, (1,2): "foo", b"asdf": 1, None: None}
 
@@ -30,7 +29,6 @@ def test_skipkeys():
     assert rj.dumps(o, skipkeys=True) == '{}'
 
 
-@pytest.mark.unit
 def test_skip_invalid_keys():
     o = {True: False, -1: 1, 1.1: 1.1, (1,2): "foo", b"asdf": 1, None: None}
 
@@ -43,7 +41,6 @@ def test_skip_invalid_keys():
     assert rj.Encoder(skip_invalid_keys=True)(o) == '{}'
 
 
-@pytest.mark.unit
 def test_ensure_ascii(dumps):
     s = '\N{GREEK SMALL LETTER ALPHA}\N{GREEK CAPITAL LETTER OMEGA}'
     assert dumps(s) == '"\\u03B1\\u03A9"'
@@ -51,7 +48,6 @@ def test_ensure_ascii(dumps):
     assert dumps(s, ensure_ascii=False) == '"%s"' % s
 
 
-@pytest.mark.unit
 def test_allow_nan():
     f = [1.1, float("inf"), 2.2, float("nan"), 3.3, float("-inf"), 4.4]
     expected = '[1.1,Infinity,2.2,NaN,3.3,-Infinity,4.4]'
@@ -91,7 +87,6 @@ def test_allow_nan():
         rj.loads(s, allow_nan=False)
 
 
-@pytest.mark.unit
 def test_native(dumps, loads):
     f = [-1, 1, 1.1, -2.2]
     expected = '[-1,1,1.1,-2.2]'
@@ -115,7 +110,6 @@ def test_native(dumps, loads):
                  parse_mode=rj.PM_COMMENTS | rj.PM_TRAILING_COMMAS) == expected
 
 
-@pytest.mark.unit
 def test_parse_mode(dumps, loads):
     trailing_comma = '[-1,1,1.1,-2.2,]'
     expected = [-1,1,1.1,-2.2]
@@ -153,7 +147,6 @@ def test_parse_mode(dumps, loads):
     assert loads(c_and_tc, parse_mode=rj.PM_COMMENTS | rj.PM_TRAILING_COMMAS) == expected
 
 
-@pytest.mark.unit
 def test_indent(dumps):
     o = {"a": 1, "z": 2, "b": 3}
     expected1 = '{\n    "a": 1,\n    "z": 2,\n    "b": 3\n}'
@@ -171,15 +164,34 @@ def test_indent(dumps):
         expected6)
 
     assert dumps(o, indent=4) in expected
+    assert dumps(o, indent="    ") in expected
+    assert dumps(o, indent="\t") in (e.replace("    ", "\t") for e in expected)
 
     with pytest.raises(TypeError):
-        dumps(o, indent="\t")
+        dumps(o, indent="\n\t")
+
+    with pytest.raises(TypeError):
+        dumps(o, indent="foo")
 
     with pytest.raises(TypeError):
         dumps(o, indent=-1)
 
 
-@pytest.mark.unit
+def test_write_mode(dumps):
+    o = {"a": 1, "b": [2, 3, 4]}
+    expected_compact = ('{"a":1,"b":[2,3,4]}', '{"b":[2,3,4],"a":1}')
+    expected_pretty = ('{\n    "a": 1,\n    "b": [2, 3, 4]\n}',
+                       '{\n    "b": [2, 3, 4],\n    "a": 1\n}')
+
+    assert dumps(o, write_mode=rj.WM_COMPACT) in expected_compact
+    assert dumps(o, indent=0, write_mode=rj.WM_COMPACT) in expected_compact
+    assert dumps(o, write_mode=rj.WM_SINGLE_LINE_ARRAY) in expected_pretty
+    assert dumps(o, write_mode=rj.WM_PRETTY|rj.WM_SINGLE_LINE_ARRAY) in expected_pretty
+
+    with pytest.raises(ValueError):
+        dumps(o, write_mode=4)
+
+
 def test_sort_keys(dumps):
     o = {"a": 1, "z": 2, "b": 3}
     expected0 = '{\n"a": 1,\n"b": 3,\n"z": 2\n}'
@@ -195,7 +207,6 @@ def test_sort_keys(dumps):
     assert dumps(o, sort_keys=True) == '{"a":"a","a\\u0000b":"a\\u0000b","a0":"a0","a1":"a1"}'
 
 
-@pytest.mark.unit
 def test_default():
     class Bar:
         pass
@@ -221,7 +232,6 @@ def test_default():
         rj.dumps(o)
 
 
-@pytest.mark.unit
 def test_decimal(dumps, loads):
     import math
     from decimal import Decimal
@@ -241,7 +251,6 @@ def test_decimal(dumps, loads):
     assert loads(dumps(d, number_mode=rj.NM_DECIMAL)) == float(dstr)
 
 
-@pytest.mark.unit
 def test_datetime_mode_dumps(dumps):
     import pytz
 
@@ -307,8 +316,13 @@ def test_datetime_mode_dumps(dumps):
                           | rj.DM_ONLY_SECONDS)
     ) == str(timegm(d.timetuple()))
 
+    # This value caused a truncation problem when we were using
+    # Writer.SetMaxDecimalPlaces(6) to emit the floating point number
+    d = datetime.fromtimestamp(1514893636.276703)
+
+    assert dumps(d, datetime_mode=rj.DM_UNIX_TIME) == str(d.timestamp())
+
 
-@pytest.mark.unit
 def test_datetime_mode_loads(dumps, loads):
     import pytz
 
@@ -339,7 +353,6 @@ def test_datetime_mode_loads(dumps, loads):
     assert load_as_naive == local.replace(tzinfo=None)
 
 
-@pytest.mark.unit
 @pytest.mark.parametrize(
     'value', [date.today(), datetime.now(), time(10,20,30)])
 def test_datetime_values(value, dumps, loads):
@@ -351,7 +364,42 @@ def test_datetime_values(value, dumps, loads):
     assert loaded == value
 
 
-@pytest.mark.unit
+@pytest.mark.parametrize(
+    'value, expected', [
+        ('1999-01-03T10:20:30.1',
+         datetime(1999, 1, 3, 10, 20, 30, 100000)),
+        ('2000-02-03T10:20:30.12',
+         datetime(2000, 2, 3, 10, 20, 30, 120000)),
+        ('2020-03-03T10:20:30.123',
+         datetime(2020, 3, 3, 10, 20, 30, 123000)),
+        ('2021-04-03T10:20:30.1234',
+         datetime(2021, 4, 3, 10, 20, 30, 123400)),
+        ('1999-08-03T10:20:30.12345',
+         datetime(1999, 8, 3, 10, 20, 30, 123450)),
+        ('1999-09-03T10:20:30.123456',
+         datetime(1999, 9, 3, 10, 20, 30, 123456)),
+        ('1999-10-03T10:20:30.1234567',
+         datetime(1999, 10, 3, 10, 20, 30, 123456)),
+        ('1999-11-03T10:20:30.12345678',
+         datetime(1999, 11, 3, 10, 20, 30, 123456)),
+        ('1999-12-03T10:20:30.123456789',
+         datetime(1999, 12, 3, 10, 20, 30, 123456)),
+        ('1999-12-03T10:20:30.123456789Z',
+         datetime(1999, 12, 3, 10, 20, 30, 123456, tzinfo=timezone.utc)),
+        ('1999-12-03T10:20:30.123456789-02:30',
+         datetime(1999, 12, 3, 10, 20, 30, 123456,
+                  tzinfo=timezone(timedelta(days=-1, seconds=77400)))),
+        ('1999-12-03T10:20:30.123456789+02:30',
+         datetime(1999, 12, 3, 10, 20, 30, 123456,
+                  tzinfo=timezone(timedelta(seconds=9000)))),
+    ]
+)
+def test_datetime_fractional_values(value, expected, dumps, loads):
+    dumped = dumps(value, datetime_mode=rj.DM_ISO8601)
+    loaded = loads(dumped, datetime_mode=rj.DM_ISO8601)
+    assert loaded == expected
+
+
 def test_uuid_mode(dumps, loads):
     assert rj.UM_NONE == 0
     assert rj.UM_CANONICAL == 1
@@ -380,7 +428,6 @@ def test_uuid_mode(dumps, loads):
     assert loaded == value
 
 
-@pytest.mark.unit
 def test_uuid_and_datetime_mode_together(dumps, loads):
     value = [date.today(), uuid.uuid1()]
     dumped = dumps(value,
@@ -392,7 +439,6 @@ def test_uuid_and_datetime_mode_together(dumps, loads):
     assert loaded == value
 
 
-@pytest.mark.unit
 @pytest.mark.parametrize(
     'value,cls', [
         ('x999-02-03', str),
@@ -428,20 +474,44 @@ def test_uuid_and_datetime_mode_together(dumps, loads):
 
         ('1999-02-03T10:20:30', datetime),
         ('1999-02-03T10:20:30Z', datetime),
+        ('1999-02-03T10:20:30.1', datetime),
+        ('1999-02-03T10:20:30.12', datetime),
         ('1999-02-03T10:20:30.123', datetime),
         ('1999-02-03T10:20:30.123Z', datetime),
+        ('1999-02-03T10:20:30.1234', datetime),
+        ('1999-02-03T10:20:30.12345', datetime),
+        ('1999-02-03T10:20:30.12345Z', datetime),
+        ('1999-02-03T10:20:30.12345+05:00', datetime),
         ('1999-02-03T10:20:30-05:00', datetime),
         ('1999-02-03T10:20:30.123456', datetime),
         ('1999-02-03T10:20:30.123456Z', datetime),
         ('1999-02-03T10:20:30.123-05:00', datetime),
         ('1999-02-03T10:20:30.123456-05:00', datetime),
+        ('1999-02-03T10:20:30.123456-05:00', datetime),
+        ('1999-02-03T10:20:30.1234567', datetime),
+        ('1999-02-03T10:20:30.12345678', datetime),
+        ('1999-02-03T10:20:30.123456789', datetime),
+        ('1999-02-03T10:20:30.1234567Z', datetime),
+        ('1999-02-03T10:20:30.12345678Z', datetime),
+        ('1999-02-03T10:20:30.123456789Z', datetime),
+        ('1999-02-03T10:20:30.1234567-20:10', datetime),
+        ('1999-02-03T10:20:30.12345678+20:11', datetime),
+        ('1999-02-03T10:20:30.123456789Z', datetime),
+        ('1999-02-03T10:20:30.123456789-01:01', datetime),
+        ('1999-02-03T10:20:30.123456789+00:00', datetime),
+        ('1999-02-03T10:20:30.', str),
+        ('1999-02-03T10:20:30.1A', str),
+        ('1999-02-03T10:20:30. ', str),
+        ('1999-02-03T10:20:30.123456789A', str),
+        ('1999-02-03T10:20:30.123456789 ', str),
+        ('1999-02-03T10:20:30.12345+00:00Z ', str),
+        ('1999-02-03T10:20:30.1A34X6+01:00Z', str),
     ])
 def test_datetime_iso8601(value, cls, loads):
     result = loads('"%s"' % value, datetime_mode=rj.DM_ISO8601)
     assert isinstance(result, cls)
 
 
-@pytest.mark.unit
 @pytest.mark.parametrize(
     'value,cls', [
         ('7a683da49aa011e5972e3085a99ccac7', str),
@@ -455,7 +525,6 @@ def test_uuid_canonical(value, cls, loads):
     assert isinstance(result, cls), type(result)
 
 
-@pytest.mark.unit
 @pytest.mark.parametrize(
     'value,cls', [
         ('za683da49aa011e5972e3085a99ccac7', str),
@@ -468,7 +537,6 @@ def test_uuid_hex(value, cls, loads):
     assert isinstance(result, cls), type(result)
 
 
-@pytest.mark.unit
 def test_object_hook():
     class Foo:
         def __init__(self, foo):
@@ -493,7 +561,6 @@ def test_object_hook():
     assert res.foo == "bar"
 
 
-@pytest.mark.unit
 @pytest.mark.parametrize(
     'posargs,kwargs', (
         ( (), {} ),
@@ -518,13 +585,12 @@ def test_object_hook():
 def test_invalid_loads_params(posargs, kwargs, loads):
     try:
         loads(*posargs, **kwargs)
-    except (TypeError, ValueError) as e:
+    except (TypeError, ValueError):
         pass
     else:
         assert False, "Expected either a TypeError or a ValueError"
 
 
-@pytest.mark.unit
 @pytest.mark.parametrize(
     'posargs,kwargs', (
         ( (), {} ),
@@ -544,13 +610,12 @@ def test_invalid_loads_params(posargs, kwargs, loads):
 def test_invalid_dumps_params(posargs, kwargs, dumps):
     try:
         dumps(*posargs, **kwargs)
-    except (TypeError, ValueError) as e:
+    except (TypeError, ValueError):
         pass
     else:
         assert False, "Expected either a TypeError or a ValueError"
 
 
-@pytest.mark.unit
 def test_explicit_defaults_loads():
     assert rj.loads(
         string='"foo"',
@@ -563,7 +628,6 @@ def test_explicit_defaults_loads():
     ) == "foo"
 
 
-@pytest.mark.unit
 def test_explicit_defaults_load():
     assert rj.load(
         stream=io.StringIO('"foo"'),
@@ -577,7 +641,6 @@ def test_explicit_defaults_load():
     ) == "foo"
 
 
-@pytest.mark.unit
 def test_explicit_defaults_decoder():
     assert rj.Decoder(
         number_mode=None,
@@ -586,7 +649,6 @@ def test_explicit_defaults_decoder():
         parse_mode=None)('"foo"') == "foo"
 
 
-@pytest.mark.unit
 def test_explicit_defaults_dumps():
     assert rj.dumps(
         obj='foo',
@@ -602,7 +664,6 @@ def test_explicit_defaults_dumps():
     ) == '"foo"'
 
 
-@pytest.mark.unit
 def test_explicit_defaults_dump():
     stream = io.StringIO()
     assert rj.dump(
@@ -621,7 +682,6 @@ def test_explicit_defaults_dump():
     assert stream.getvalue() == '"foo"'
 
 
-@pytest.mark.unit
 def test_explicit_defaults_encoder():
     assert rj.Encoder(
         skip_invalid_keys=False,
@@ -633,7 +693,6 @@ def test_explicit_defaults_encoder():
         uuid_mode=None)({'foo': 'bar'}) == '{"foo":"bar"}'
 
 
-@pytest.mark.unit
 def test_encoder_call():
     o = {'foo': 'bar'}
 
@@ -648,3 +707,37 @@ def test_encoder_call():
     stream = io.StringIO()
     assert rj.Encoder()(o, stream=stream) is None
     assert stream.getvalue() == result
+
+
+def test_decoder_attrs():
+    d = rj.Decoder(
+        number_mode=rj.NM_NAN,
+        datetime_mode=rj.DM_ISO8601,
+        uuid_mode=rj.UM_CANONICAL,
+        parse_mode=rj.PM_COMMENTS)
+    assert d.number_mode == rj.NM_NAN
+    assert d.datetime_mode == rj.DM_ISO8601
+    assert d.uuid_mode == rj.UM_CANONICAL
+    assert d.parse_mode == rj.PM_COMMENTS
+
+
+def test_encoder_attrs():
+    e = rj.Encoder(
+        skip_invalid_keys=True,
+        ensure_ascii=True,
+        indent='\n',
+        sort_keys=True,
+        number_mode=rj.NM_NAN,
+        datetime_mode=rj.DM_ISO8601,
+        uuid_mode=rj.UM_CANONICAL,
+        bytes_mode=rj.BM_UTF8)
+    assert e.skip_invalid_keys
+    assert e.ensure_ascii
+    assert e.indent_char == '\n'
+    assert e.indent_count == 1
+    assert e.write_mode == rj.WM_PRETTY
+    assert e.sort_keys
+    assert e.number_mode == rj.NM_NAN
+    assert e.datetime_mode == rj.DM_ISO8601
+    assert e.uuid_mode == rj.UM_CANONICAL
+    assert e.bytes_mode == rj.BM_UTF8
